package com.fuyuanshen.app.service.impl;

import com.fuyuanshen.common.core.utils.MapstructUtils;
import com.fuyuanshen.common.mybatis.core.page.TableDataInfo;
import com.fuyuanshen.common.mybatis.core.page.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import com.fuyuanshen.app.domain.bo.AppRoleMenuBo;
import com.fuyuanshen.app.domain.vo.AppRoleMenuVo;
import com.fuyuanshen.app.domain.AppRoleMenu;
import com.fuyuanshen.app.mapper.AppRoleMenuMapper;
import com.fuyuanshen.app.service.IAppRoleMenuService;

import java.util.List;
import java.util.Map;
import java.util.Collection;

/**
 * 角色和菜单关联Service业务层处理
 *
 * @author Lion Li
 * @date 2025-06-27
 */
@Slf4j
@RequiredArgsConstructor
@Service
public class AppRoleMenuServiceImpl implements IAppRoleMenuService {

    private final AppRoleMenuMapper baseMapper;

    /**
     * 查询角色和菜单关联
     *
     * @param roleId 主键
     * @return 角色和菜单关联
     */
    @Override
    public AppRoleMenuVo queryById(Long roleId){
        return baseMapper.selectVoById(roleId);
    }

    /**
     * 分页查询角色和菜单关联列表
     *
     * @param bo        查询条件
     * @param pageQuery 分页参数
     * @return 角色和菜单关联分页列表
     */
    @Override
    public TableDataInfo<AppRoleMenuVo> queryPageList(AppRoleMenuBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<AppRoleMenu> lqw = buildQueryWrapper(bo);
        Page<AppRoleMenuVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询符合条件的角色和菜单关联列表
     *
     * @param bo 查询条件
     * @return 角色和菜单关联列表
     */
    @Override
    public List<AppRoleMenuVo> queryList(AppRoleMenuBo bo) {
        LambdaQueryWrapper<AppRoleMenu> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<AppRoleMenu> buildQueryWrapper(AppRoleMenuBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<AppRoleMenu> lqw = Wrappers.lambdaQuery();
        lqw.orderByAsc(AppRoleMenu::getRoleId);
        lqw.orderByAsc(AppRoleMenu::getMenuId);
        return lqw;
    }

    /**
     * 新增角色和菜单关联
     *
     * @param bo 角色和菜单关联
     * @return 是否新增成功
     */
    @Override
    public Boolean insertByBo(AppRoleMenuBo bo) {
        AppRoleMenu add = MapstructUtils.convert(bo, AppRoleMenu.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setRoleId(add.getRoleId());
        }
        return flag;
    }

    /**
     * 修改角色和菜单关联
     *
     * @param bo 角色和菜单关联
     * @return 是否修改成功
     */
    @Override
    public Boolean updateByBo(AppRoleMenuBo bo) {
        AppRoleMenu update = MapstructUtils.convert(bo, AppRoleMenu.class);
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(AppRoleMenu entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 校验并批量删除角色和菜单关联信息
     *
     * @param ids     待删除的主键集合
     * @param isValid 是否进行有效性校验
     * @return 是否删除成功
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteByIds(ids) > 0;
    }
}
